module Schemas
  class Base

    include Dry::Schema

    def normalize_integer(value)
      return nil if value.blank?
      Integer(value.to_s.gsub(/\s/, '')) rescue value
    end


    def strip_string(value, empty_to_nil = true)
      value = value.to_s.strip
      value = nil if empty_to_nil && value.blank?
      value
    end


    def to_date(value)
      value.to_date rescue value
    end


    def clear_html(value)
      Rails::Html::WhiteListSanitizer.new.sanitize(value)
    end


    def normalize_url(value)
      return if value.blank?
      value.to_s.strip.scan(/^https?:\/\//).present? ? value : sprintf('https://%s', value.to_s.strip)
    end


    def to_bool(value)
      value.to_s.to_bool
    end


    def to_float(value)
      return unless value.present?
      BigDecimal.new(value.to_s.gsub(/\s/, '')) rescue value
    end


    def to_big_decimal(value)
      return unless value.present?
      BigDecimal.new(value.to_s.gsub(/\s/, '')) rescue value
    end


    def forced_array(value)
      [value.present? ? value : nil].flatten.compact
    end


    def forced_hash(value)
      value.blank? ? {} : value
    end


    def nullify_empty(value)
      value.present? ? value : nil
    end

  end
end
