module Schemas
  module CentralAgent
    class SaveBoatSchema < Schemas::BaseBoatSchema
      def schema
        boat_schema = super
        boat_schema[:required] = %i[
          name
          boat_type_id
        ]

        boat_schema[:properties][:boat_photos_attributes] = {
            type: [:null, :hash],
            cast: ->(value) { nullify_empty(value) }
        }

        # второй вариант реализации property boat_length, который присутствует в базовой схеме
        %i[boat_length_metrics boat_beam_metrics boat_draft_metrics].product([:meters, :ft]).map { |f,d| '%s_%s' % [f,d] }.each do |prop|
          boat_schema[:properties][prop.to_sym] = {
              type: %i[null string number],
              cast: ->(value) do
                val = value.is_a?(String) ? value.split(/[, ]/).join : value
                to_float val
              end
          }
        end

        boat_schema
      end


      def validate_additional
        validate_boat_photos
        validate_prices_and_rent_prices
        validate_rent_prices
        validate_length
        validate_location
        validate_guests_total
        # validate_guest_cabins
        validate_crew_total
      end

      def validate_boat_photos
        errors.add :boat_photos, I18n.t('errors.boat.boat_photos_attributes.blank') unless boat_photos_attributes.present?
      end


      def validate_length
        val = boat_length_metrics_meters || boat_length_metrics_ft
        unless val.present? && val > 0 && val < 1000
          errors.add(:boat_length_metrics, I18n.t('errors.messages.invalid'))
        end
      end
    end
  end
end