module Schemas
  module Site
    class RegisterBoatSchema < Schemas::BaseBoatSchema

      def schema
        super_schema = super

        super_schema[:required] = %i[
                name
                boat_type_id
                boat_model
                builder
                length
                crew_total
                guest_cabins
                guests_total
                maximum_guests_during_cruise
                boat_price
                boat_sale_price
        ]

        super_schema[:properties][:boat_price] = {
            type: :hash,
            required: %i[duration uom_id season_id currency_id value],
            properties: {
                duration: {
                    type: :number,
                    cast: ->(value) { normalize_integer(value) }
                },
                uom_id: {
                    enum: Dicts::CharterUom.all.collect(&:id),
                    cast: ->(value) { normalize_integer(value) }
                },
                season_id: {
                    enum: Dicts::CharterSeason.all.collect(&:id),
                    cast: ->(value) { normalize_integer(value) }
                },
                currency_id: {
                    enum: Dicts::Currency.all.collect(&:id),
                    cast: ->(value) { normalize_integer(value) }
                },
                value: {
                    type: %i[number nil],
                    cast: ->(value) { normalize_integer(value) }
                }
            }
        }

        super_schema[:properties][:boat_sale_price] = {
            type: :hash,
            required: %i[value currency_id],
            properties: {
                currency_id: {
                    enum: Dicts::Currency.all.collect(&:id),
                    cast: ->(value) { normalize_integer(value) }
                },
                value: {
                    type: %i[number nil],
                    cast: ->(value) { normalize_integer(value) }
                }
            }
        }

        super_schema
      end

      def validate_additional
        validate_prices
        validate_location
        validate_length
        # validate_crew_total
        # validate_guest_cabins
        validate_guests_total
      end

      # noinspection RubyResolve
      def validate_prices
        unless boat_price[:value]&.nonzero? || boat_sale_price[:value]&.nonzero?
          errors.add :prices, I18n.t('errors.boat.prices.any')
        end
      end

    end
  end
end