module Accounts
  class ClientAccountRepository

    def fetch_or_create(user)
      return unless user
      return user.accounts.clients.first.client if user.accounts.clients.any?
      create_account(user)
    end

    def create_account(user)
      return unless user
      return if user.accounts.clients.any?

      account = ::Account.new(type_id: ::Dictionaries::AccountType::CLIENT)
      client  = ::Accounts::Client.new(account: account)

      account.user   = user
      account.name   = user.name.to_s
      account.active = true

      account.transaction do
        account.save validate: false
        client.save validate: false
      end

      client
    end

    def destroy_account(user)
      return unless user
      return unless user.accounts.clients.any?

      account = user.accounts.clients.first
      client  = account.client

      ActiveRecord::Base.transaction do
        client.destroy
        account.destroy
      end
    end

  end
end
