class AccountSerializer < AbstractSerializer
  class << self
    def available_attributes
      %i[
        active
        created_at
        id
        type_index
        props
        abilities
      ]
    end

    def active(account)
      { active: account.active }
    end

    def created_at(account)
      { created_at: account.created_at }
    end

    def id(account)
      { id: account.id }
    end

    def type_index(account)
      { type_index: ::Dicts::AccountType.find(account.type_id).index }
    end

    def props(account)
      props = case account.type_id
                when ::Dicts::AccountType::MANAGER_GUEST.id
                  mg = account.manager_guest
                  {
                      radius: mg.radius.to_i
                  }
                else
                  {}
              end
      { props: props }
    end

    # список возможностей с учётом запретов
    def abilities(account)
      account_class     = ::Dictionaries::AccountType::CLASS_BY_TYPE[account.type_id]
      default_abilities = ::Dicts::Accounts::Ability.all_for_account_class(account_class)
      prohibited        = account.user.permissions

      # отнимаем запрещённые действия из дефолтных
      not_prohibited = ->(ability) { prohibited.none? { |a| a.action.to_s == ability.index.to_s } }
      abilities      = default_abilities.map do |a|
        {
          action: a.index,
          can:    not_prohibited[a]
        }
      end

      { abilities: abilities }
    end
  end
end
