class BoatSerializer < AbstractSerializer

  class << self
    def available_attributes
      %i[
        amenities
        boat_beam
        boat_beam_metrics
        boat_contacts
        boat_cruise_speed
        boat_draft
        boat_draft_metrics
        boat_engines
        boat_hull_id
        boat_generators
        boat_length
        boat_length_metrics
        boat_length_metrics_ft
        boat_length_metrics_meters
        boat_locations_attributes
        boat_max_speed
        boat_model
        boat_photos
        boat_photos_attributes
        boat_range
        boat_stars
        boat_thumbups
        boat_type
        boat_type_id
        builder
        built_year
        charter_details
        charter_deposit
        crew_total
        crew_cabins
        description
        engine_hours
        exterior_color
        equipment
        for_rent
        for_sale
        fuel_type
        guest_cabins
        guest_heads
        guests_total
        hull_material
        id
        interior_designer
        length
        latitude
        location_address
        locations
        longitude
        maximum_guests_during_cruise
        main_boat_photo_id
        motor
        motor_id
        name
        naval_architect
        picture_medium_url
        prices
        refit_year
        rent_prices
        rental_type
        reviews
        safety
        sale_price
        short_description
        state
        toys
        visits_today
        visits_total
      ].freeze
    end

    def default_opts
      {
        uol:         'ft',
        currency:    ::Dicts::Currency::USD.index,
        locale:      'en',
        is_for_rent: true,  # TODO:: [bcff00da] УДАЛИТЬ
        business:    ::Dicts::Business::RENT.index
      }
    end

    def serialize(model, attributes: available_attributes, opts: {})
      result = super(model, attributes: attributes, opts: default_opts.merge(opts))
      result[:opts] = serialize_opts
      result
    end

    # определяем "простые" методы
    %i[id comments tender name description built_year boat_model boat_type_id boat_hull_id refit_year
       crew_cabins crew_total guest_heads maximum_guests_during_cruise interior_designer
       naval_architect guests_total guest_cabins visits_today visits_total
       boat_cruise_speed boat_max_speed boat_range boat_engines boat_generators engine_hours].each do |f|
      define_method f do |boat|
        { f => boat.send(f) }
      end
    end

    def state(boat)
      { state: I18n.t('attributes.state_%s' % boat.state) }
    end

    def toys_attributes
      %i[
        banana
        beach_kids_toys
        donut
        exercise_equipment
        fishing_gear
        flyboard
        inflatable_sailboat
        jet_ski
        kayak
        kneeboard
        paddle_boards
        sailing_boat
        scuba_tanks
        skiboard
        snorkelling_gear
        stand_up_jet_ski
        underwater_gear
        wakeboard
        water_ski
        windsurfing_board
      ].freeze
    end

    def toys(boat)
      {
        toys: toys_attributes.map do |f|
          [f, boat.send(f)]
        end.to_h
      }
    end

    def equipment_attributes
      %i[
        apple_tv
        autopilot
        cd_player
        chartplotter
        cockpit_loudspeakers
        entertainment
        excellent_sound_system
        gps
        internet
        lcd_tv
        navigation
        radar
        satellite_internet
        satellite_phone
        tv
        vhf
        wi_fi
      ].freeze
    end

    def amenities_attributes
      %i[
        air_conditioner
        bow_thruster
        electric_toilet
        essentials
        family_kid_friendly
        gym
        hardtop
        heating
        helipad
        hot_cold_cockpit_shower
        jacuzzi_on_deck
        kitchen
        pet_allowed
        sauna
        soft_top
        stabilisers_underway
        stabilizers_at_anchor
        steam_room
        toilet
      ].freeze
    end

    def safety_attributes
      %i[
        carbon_detector
        epirb
        fire_extinguisher
        first_aid_kit
        flares
        horn
        lifebuoy
        life_jackets
        raft
        smoke_detector
        vessel_lighting
        visual_distress_signals
      ].freeze
    end

    def equipment(boat)
      {
          equipment: equipment_attributes.map do |f|
            [f, boat.send(f)]
          end.to_h
      }
    end

    def amenities(boat)
      {
          amenities: amenities_attributes.map do |f|
            [f, boat.send(f)]
          end.to_h
      }
    end

    def safety(boat)
      {
          safety: safety_attributes.map do |f|
            [f, boat.send(f)]
          end.to_h
      }
    end

    # noinspection RubyResolve
    def picture_medium_url(boat)
      url = if boat.boat_photo.present?
              boat.boat_photo.picture.url(:medium)
            else
              ''
            end

      { picture_medium_url: url }
    end

    def motor(boat)
      m = ::Dicts::Motor.find boat.motor
      {
          motor: {
              id:     (m.id    rescue nil),
              index:  (m.index rescue 'null')                                                                           # 'null' - индекс в локалях
          }
      }
    end

    def main_boat_photo_id(boat)
      boat_photo_id = if boat.boat_photo.present?
                        boat.boat_photo.id
                      end

      { main_boat_photo_id: boat_photo_id }
    end

    def builder(boat)
      { builder: boat.builder&.strip }
    end

    def short_description(boat)
      attr  = 'short_description_%s' % @opts[:locale]
      value = boat.send(attr.to_sym)
      { short_description: value }
    end

    def exterior_color(boat)
      color = ::Dicts::ExteriorColor.find boat.exterior_color
      {
          exterior_color: {
              id:     (color.id    rescue nil),
              index:  (color.index rescue 'null')                                                                       # 'null' - индекс в локалях
          }
      }
    end

    # per day, per week; в день, в час, за месяц;
    # def per_season(boat)
    #   season_key = ::Lib::Boats::RentPricePerSeason.spot_rentprice_id(boat, @opts[:currency], @opts[:rent_price_id]).gsub(/_low|_high/, '')
      #=> charter_1_week_low, charter_1_day_high
      # translate  = I18n.t("boat.rent_price.per_#{season_key}")
      # value      = @opts[:is_for_rent] ? translate : ''
      #
      # { per_season: value }
    # end

    # ₽ 293,885
    # def price(boat)
    #   value = if @opts[:is_for_rent]
    #             ::Lib::Boats::RentPricePerSeason.price(boat, @opts[:currency], @opts[:rent_price_id])
    #           else
    #             ::Lib::Boats::SalePrice.price(boat, @opts[:currency])
    #           end
    #
    #   { price: value }
    # end

    # 12%
    # def price_discount(boat)
    #   value = if @opts[:is_for_rent]
    #             ::Lib::Boats::RentPricePerSeason.price_discount(boat, @opts[:currency])
    #           else
    #             ::Lib::Boats::SalePrice.price_discount(boat)
    #           end
    #   { price_discount: value }
    # end

    def hull_material(boat)
      hm = ::Dicts::HullMaterial.find boat.hull_material
      {
          hull_material: {
              id:     (hm.id rescue nil),
              index:  (hm.index rescue 'null')                                                                          # 'null' - индекс в локалях
          }
      }
    end

    def rental_type(boat)
      hm = ::Dicts::RentSkipType.find boat.rental_type
      {
          rental_type: {
              id:     (hm.id rescue nil),
              index:  (hm.index rescue 'null')                                                                          # 'null' - индекс в локалях
          }
      }
    end

    def fuel_type(boat)
      {
        fuel_type: boat.fuel_type
      }
    end

    def boat_type(boat)
      {
          boat_type: {
              id:     boat.boat_type_id,
              title:  boat.boat_type&.name&.strip
          }
      }
    end

    # для формы редактирования в кабинете агента
    def boat_beam_metrics(boat)
      {
          boat_beam_metrics: {
              value: boat.send('boat_beam_metrics_%s' % @opts[:uol]),
              uol:   @opts[:uol]
          }
      }
    end

    # для формы редактирования в кабинете агента
    def boat_draft_metrics(boat)
      {
          boat_draft_metrics: {
              value: boat.send('boat_draft_metrics_%s' % @opts[:uol]),
              uol:   @opts[:uol]
          }
      }
    end

    # используется в списке лодок агента, в preview-блоке на сайте
    def boat_length(boat)
      {
          boat_length: {
              value: boat.send('boat_length_metrics_%s' % @opts[:uol]),
              uol:   @opts[:uol]
          }
      }
    end

    # используется для валидации лодки при обновлении стейта
    def length(boat)
      {
          length: {
              value:     boat.send('boat_length_metrics_%s' % @opts[:uol]),
              length_id: ::Dicts::Length.find_by_index(@opts[:uol]).id
          }
      }
    end

    # для формы редактирования в кабинете агента
    def boat_length_metrics(boat)
      { boat_length_metrics: boat_length(boat)[:boat_length] }
    end

    # [bcff00da] : вся подобная логика переезжает во фронт, переделано должно быть по образу `def boat_length`

    # TODO:: [bcff00da] УДАЛИТЬ - пока используется при обновлении/создании лодки в админке
    def boat_length_metrics_meters(boat)
      { boat_length_metrics_meters: boat.boat_length_metrics_meters }
    end

    # TODO:: [bcff00da] УДАЛИТЬ - пока используется при обновлении/создании лодки в админке
    def boat_length_metrics_ft(boat)
      { boat_length_metrics_ft: boat.boat_length_metrics_ft }
    end

    def prices(boat)
      currency_id  = ::Dicts::Currency.find_by_index(@opts[:currency]).id
      business     = ::Dicts::Business.find_by_index @opts[:business]

      prices       = if business.index == ::Dicts::Business::RENT.index
                       boat.boat_prices.select { |bp| bp.currency_id == currency_id }.map { |bp| ::BoatPriceSerializer.serialize bp }
                     else
                       # noinspection RubyResolve
                       boat.boat_sale_prices.select { |bp| bp.currency_id == currency_id }.map { |bps| ::BoatSalePriceSerializer.serialize bps }
                     end
      {
          prices:   {
              list:     prices,
              business: {
                  index: business.index,
                  id:    business.id
              }
          }
      }
    end

    def rent_prices(boat)
      currency_id = ::Dicts::Currency.find_by_index(@opts[:currency]).id

      {
          rent_prices: boat.boat_prices
                           .select { |bp| bp.currency_id == currency_id }
                           .map.each_with_index { |bp, index| ::BoatPriceSerializer.serialize(bp).merge(id:index) }
      }
    end

    # noinspection RubyResolve
    def sale_price(boat)
      currency_id = ::Dicts::Currency.find_by_index(@opts[:currency]).id
      sale_price  = boat.boat_sale_prices.select { |bp| bp.currency_id == currency_id }.first

      {
          sale_price: (::BoatSalePriceSerializer.serialize sale_price rescue nil)
      }
    end

    def boat_photos(boat)
      # FIXME:: выяснить, нужно ли всётаки использовать .gsub('\'', '&#39;') для строк с путями к фоткам?
      boat_photos = boat.boat_photos.map do |boat_photo|
        {
          id:           boat_photo.id,
          gallery_path: boat_photo.picture.url(:gallery),
          medium_path:  boat_photo.picture.url(:medium),
          thumb_path:   boat_photo.picture.url(:thumb)
        }
      end
      { boat_photos: boat_photos }
    end

    # используется для валидации лодки при обновлении стейта
    def boat_photos_attributes(boat)
      {
          boat_photos_attributes: boat.boat_photos.map do |boat_photo|
            { id: boat_photo.id }
          end
      }
    end

    # Контакты владельцев: то, что приходит в `boat_users_attributes` из админской формы
    def boat_contacts(boat)
      boat_contacts = boat.boat_users.map do |user|
        {
          user_id: user.user_id
        }
      end
      { boat_contacts: boat_contacts }
    end

    # под видом этого атрибута будет приходить число - latitude главной локации
    def latitude(boat)
      loc   = boat.boat_locations.select { |bl| bl.is_main }.first
      loc ||= boat.boat_locations.first
      { latitude: loc&.latitude || 0 }
    end

    # под видом этого атрибута будет приходить число - longitude главной локации
    def longitude(boat)
      loc   = boat.boat_locations.select { |bl| bl.is_main }.first
      loc ||= boat.boat_locations.first
      { longitude: loc&.longitude || 0 }
    end

    # под видом этого атрибута будет приходить текст - название главной локации
    def location_address(boat)
      loc   = boat.boat_locations.select { |bl| bl.is_main }.first                                                      # если не применяется поиск по адресу
      loc ||= boat.boat_locations.first                                                                                 # если поиск по адресу применён -- в результат поиска может не попасть главная локация
      {
          location_address: loc&.address || ''
      }
    end

    def locations(boat)
      {
          locations: boat.boat_locations.map do |location|
            ::BoatLocationSerializer.serialize location
          end
      }
    end

    # используется для валидации лодки при обновлении стейта
    def boat_locations_attributes(boat)
      {
          boat_locations_attributes: boat.boat_locations.map do |location|
            ::BoatLocationSerializer.serialize location
          end
      }
    end

    def for_rent(boat)
      { for_rent: boat.for_rent.to_i == 1 }
    end

    def for_sale(boat)
      { for_sale: boat.for_sale.to_i == 1 }
    end

    def charter_details(boat)
      { charter_details: boat.charter_details }
    end

    def charter_deposit(boat)
      { charter_deposit: boat.charter_deposit }
    end

    # выдаем 1 самое свежее значение
    def boat_stars(boat)
      { boat_stars: boat.boat_stars.max_by(&:created_at)&.value }
    end

    # выдаем 1 самое свежее значение
    def boat_thumbups(boat)
      { boat_thumbups: boat.boat_thumbups.max_by(&:created_at)&.value }
    end

    # TODO:: impelement it
    def reviews(boat)
      { reviews: [] }
    end

    def serialize_opts
      locale   = ::Dicts::Locale.find_by_index   @opts[:locale].to_s
      uol      = ::Dicts::Length.find_by_index   @opts[:uol]
      business = ::Dicts::Business.find_by_index @opts[:business]
      currency = ::Dicts::Currency.find_by_index @opts[:currency]

      {
          locale: {
              index: locale.index,
              id:    locale.id
          },
          uol: {
              index: uol.index,
              id:    uol.id
          },
          business: {
              index: business.index,
              id:    business.id
          },
          currency: {
              index:  currency.index,
              id:     currency.id,
              symbol: currency.symbol
          }
      }
    end
  end
end
