module Lease
  class RebateSerializer < ::AbstractSerializer

    extend ::Custom::CurrencyHelper
    extend ::ActionView::Helpers::NumberHelper

    class << self

      def available_attributes
        [
          :id,
          :amount,
          :currency,
          :human_price,
          :human_price_vat_incl,
          :old_amount,
          :old_currency,
          :old_human_price,
          :old_human_price_val_exl
        ].freeze
      end

      def id(rebate)
        { id: rebate.id }
      end

      def amount(rebate)
        { amount: rebate.amount }
      end

      def currency(rebate)
        { currency: rebate.currency }
      end

      # то, что юзер ввёл в диалоговое окно - "total renting price, VAT excl."
      def human_price(rebate)
        val = nil
        if rebate.amount.present? && rebate.currency.present?
          val = num_to_cur(rebate.amount, rebate.currency, @opts[:locale])
        end
        { human_price: val }
      end

      def human_price_vat_incl(rebate)
        val = nil
        if rebate.amount.present? && rebate.currency.present?
          val = rebate.amount / 100.0 * (100 + rebate.bid.vat)
          val = num_to_cur(val.to_i, rebate.currency, @opts[:locale])
        end

        { human_price_vat_incl: val }
      end

      # VAT incl
      def old_amount(rebate)
        { old_amount: rebate.old_amount }
      end

      def old_currency(rebate)
        { old_currency: rebate.old_currency }
      end

      # VAT incl
      def old_human_price(rebate)
        val = nil
        if rebate.old_amount.present? && rebate.old_currency.present?
          val = num_to_cur(rebate.old_amount, rebate.old_currency, @opts[:locale])
        end
        { old_human_price: val }
      end

      def old_human_price_val_exl(rebate)
        val = nil
        if rebate.old_amount.present? && rebate.old_currency.present?
          val = rebate.old_amount * 100.0 / (100 + rebate.bid.vat)
          val = num_to_cur(val.to_i, rebate.old_currency, @opts[:locale])
        end
        { old_human_price_val_exl: val }
      end
    end
  end
end
