module Lib
  module Boats
    class RentPricePerSeason
      extend ::Custom::CurrencyHelper
      extend ::Custom::PriceHelper
      extend ::ActionView::Helpers::NumberHelper

      # Вернёт отформатированное минимальное число среди цен на аренду вместе с подписью.
      # Если у лодки не заполнены цены - вернёт дефлотное значение.
      # value возвращается, но не используется.
      #
      # @return [Hash] { value: String, perseason: String }
      #
      def self.min_rentprice(boat, currency)
        ::Dictionaries::BoatCharterAttribute::ALL.map do |attr|
          {
            attr:  attr.to_s,
            value: boat.send(currency_attribute_name(attr, currency))
          }
        end.delete_if { |el| !el[:value].present? }.sort_by { |el| el[:value] }.first || { attr: 'charter_1_day_high', value: 0 }
      end

      # Определить столбец, из которого надо взять цену аренды.
      # Может зависеть от контекста - от того, какой параметр выставлен в поисковой форме.
      # Если с поисковой формы приходит nil или пустая строка - вернёт дефолтное значение.
      # Если у лодки не заполнено искомое поле - вернёт дефолтное значение.
      # Дефолтное значение - вычисляется с помощью min_rent_price
      #
      # @param  [Symbol, String] rent_price_id Например: charter_1_day_low
      # @return [String]
      #
      def self.spot_rentprice_id(boat, currency = 'USD', rent_price_id = nil)
        default_id =  min_rentprice(boat, currency)[:attr]
        return default_id unless rent_price_id

        return default_id unless boat.attributes[currency_attribute_name(rent_price_id, currency)].present?

        rent_price_id.to_s
      end

      def self.price(boat, currency = 'USD', rent_price_id = nil)
        attr = spot_rentprice_id(boat, currency, rent_price_id)
        price_format(boat, attr, currency: currency)
      end

      def self.price_discount(boat, currency = 'USD')
        boat.send('%s_discount' % min_rentprice(boat, currency)[:attr])
      end
    end
  end
end
