class AbstractPricesService

  attr_reader :built_prices

  def initialize(all_currencies = ::Dicts::Currency::ALL2)                                                              # all_currencies - список валют, в которых надо сохранить цены
    @all_currencies = all_currencies
    _reset_ivars
  end

  private

  def _model
    raise NotImplementedError
  end

  def _fields_for_create
    raise NotImplementedError
  end

  def _save(prices)                                                                                                     # помещаем в базу всё одним запросом
    filtered = prices.select { |el| !el[:_delete] && el[:value].to_i != 0 }                                             # игнорируя цены, которые были помечены "к удалению" или те, у которых значение 0 (0 можно подавать, если хотим "удалить" цену продажи)
    return if filtered.size.zero?

    values = filtered.map do |p|
      _fields_for_create.map do |field|
        p[field.to_sym]
      end.compact * ','
    end * '),('
    values = '(%s)' % values
    sql    = 'INSERT INTO %s(%s) VALUES %s' % [_model.table_name, _fields_for_create * ',', values]
    ActiveRecord::Base.connection.execute sql

    filtered
  end

  # из базы удаляем все записи, принадлежащие данной лодке
  # т.к. считаем, что с формы приходит полная картина по ценам
  # кроме того, юзер может просто изменить, например, период в уже
  # существующей строке с ценами, которую он видит на экране
  def _delete_existing_records(boat_id)
    sql = 'DELETE FROM %s WHERE boat_id = %s' % [_model.table_name, boat_id]
    ActiveRecord::Base.connection.execute sql
  end

  def _build_other(price)                                                                                               # отталкиваясь от валюты входных данных, рассчитываем "те же" цены в других валютах
    return if price[:_delete] || price[:value].to_i == 0                                                                # игнорируя цены, которые были помечены "к удалению" или те, у которых значение 0 (0 можно подавать, если хотим "удалить" цену продажи)

    incoming_curr  = ::Dicts::Currency.find(price[:currency_id])
    currs_to_build = @all_currencies - [incoming_curr]
    rates          = Currency.fresh(incoming_curr.index).rates
    rates          = JSON.parse(rates)['rates']

    currs_to_build.each do |curr|
      rate               = rates[curr.index]
      res                = price.clone
      res[:currency_id]  = curr.id
      res[:value]        = res[:value].to_f * rate
      res[:is_orig]      = false
      @built_prices      << res
    end
  end

  def _delete_bad_values(prices)
    prices.delete_if { |price| price[:value].nil? || price[:value].to_f.zero? }
  end

  def _reset_ivars
    @built_prices = []
  end
end
