class BoatRegisterService

  attr_reader :boat,
              :errors,
              :user

  def initialize
    _reset_instance_variables
  end

  # @param boat_params  - параметры лодки, пришедшие с формы регистрации лодки
  # @param request      - нужен только для того, чтобы установить ip юзеру
  # @param user         - это current_user (в номинальном режиме)
  #
  def perform(boat_params, request, user = nil)
    _reset_instance_variables

    schema = ::Schemas::Site::RegisterBoatSchema.new boat_params                                                        # проверяем параметры, пришедшие с формы
    unless schema.valid?
      @errors = schema.errors.messages
      return false
    end

    _create_user(boat_params.dig(:users_attributes, '0'), user)                                                         # # затем создаём и проверяем юзера == NOTE: такой странный хэш приходит 100%, если в модели Boat есть accept_nested_attributes :users
    return false if @user.new_record? && !@user.valid?
    _new_boat boat_params                                                                                               # создаём лодку (но в базу пока не кладём)

    ActiveRecord::Base.transaction do                                                                                   # если дошли до этого этапа - значит всё ровно и с юзером и с лодкой - связываем их и пишем в базу
      @boat.users << @user
      @user.save validate: false

      @boat.send :set_slug
      @boat.save validate: false

      # Делаем первую фотку главной и пишем в базу IP
      MediaService.new(@boat).set_main_picture(:boat_photo)
      UserService.new(@user).set_ip(request.remote_ip.to_s)

      # в конце концов рассчитываем цены во всех валютах
      ::Boats::BoatPricesSaveService.perform(@boat, [schema.attributes[:boat_price]])            unless @boat.for_rent.zero?
      ::Boats::BoatSalePricesSaveService.perform(@boat.id, schema.attributes[:boat_sale_price])  unless @boat.for_sale.zero?

      # TODO:: во-первых, заменить условия в двух строках выше на анализ boat_params[:boat_price] и boat_params[:boat_sale_price] соответственно
      # TODO:: во-вторых, только после успешной отработки ценовых сервисов определять for_sale и for_rent и обновлять атрибуты

    end

    true
  end

  private

  # noinspection RubySimplifyBooleanInspection,RubyResolve
  def _new_boat(boat_params)
    boat_locs2          = boat_params[:boat_locations_attributes].compact.each_with_index.map { |attr, index| [index.to_s, attr] }.to_h
    boat_new_params     = boat_params.except(:users_attributes, :prices, :boat_sale_price, :boat_price, :length, :boat_locations_attributes)    # это собираемся подать в Boat.new, но сначала допишем длину
    lk                  = Dicts::Length.find(boat_params[:length][:length_id]).index                                                            # "meters", "ft"
    lk                  = 'boat_length_metrics_%s' % lk
    boat_new_params[lk] = boat_params[:length][:value]

    boat_new_params[:boat_locations_attributes] = boat_locs2

    # fuckin' active record
    @boat = Boat.new
    @boat.assign_attributes boat_new_params.permit(
        :boat_type_id, :boat_model, :name,
        :builder,      :crew_total, :guest_cabins, :guests_total,
        :maximum_guests_during_cruise,
        # :longitude,    :latitude,   :location_address,
        lk.to_sym,
        boat_photos_attributes:    %i[id boat_id name picture _destroy],
        boat_locations_attributes: %i[latitude longitude address is_main radius _destroy id]
    )
    @boat.premoderate

    length_index = ::Dicts::Length.find(boat_params[:length][:length_id]).index                                         # вместо cookies[:dimension]
    DimensionService.new(@boat).calculate_boat_attributes length_index

    @boat.for_sale = !!boat_params[:boat_sale_price][:value]&.to_i&.nonzero?
    @boat.for_rent = !!boat_params[:boat_price][:value]&.to_i&.nonzero?

    true
  end

  def _create_user(user_params, user = nil)
    @user = user || User.new(user_params.permit(:email, :password))
    return unless @user.new_record?                          # 2. если пользователь авторизован

    email    = user_params[:email]
    password = user_params[:password]

    # 1. если пользователь не авторизован
    if email.present? && password.present?                   # 1.1 если указаны email и password - поищем юзера в базе
      may_be_user = User.find_by_email email
      if may_be_user                                         # 1.1.1 если есть такой юзер в базе - проверим правильность пароля
        if may_be_user.valid_password? password              # 1.1.1.1 если пароль правильный
          @user = may_be_user
          return
        else                                                 # 1.1.1.2 если пароль неправильный -- сообщаем об этом
          errors[:users_attributes_0_email] =                # note : такой странный ключ хэша обусловлен Boat.accept_nested_attributes :user
            [I18n.t('devise.failure.invalid', authentication_keys: 'email')]
          return
        end
      else                                                   # 1.1.2 если такого юзера нет в базе - СОЗДАЁМ
        @user.role_id  = 11                                  # todo-my:: 2018-08-06: потом это будет заменено на создание аккаунта Central Agent (просто сейчас находимся в контексте разработки list your boat)
        @user.name     = @user.email.split('@')[0]
        return
      end
    else                                                     # 1.2 если не указан или пароль или емайл - сообщаем об этом
      errors[:users_attributes_0_email] =                    # note : такой странный ключ хэша обусловлен Boat.accept_nested_attributes :user
        [I18n.t('devise.failure.invalid', authentication_keys: 'email')]
    end

  end

  def _reset_instance_variables
    @errors = { }
  end

end
