module Boats
  #
  # spec/services/boat/boat_prices_save_service_spec.rb
  #
  class BoatPricesSaveService < ::AbstractPricesService

    attr_reader :is_for_rent

    def self.perform(boat, params)
      new.perform(boat, params)
    end

    def perform(boat, prices = nil)
      _reset_ivars
      return false if prices.nil?

      boat_prices_params = prices.deep_dup                                                                              # Rails deep_dup https://apidock.com/rails/Hash/deep_dup
      boat_prices_params = boat_prices_params.map { |bp| bp.symbolize_keys }
      filtered           = _delete_bad_values boat_prices_params
      return false if filtered.size.zero?                                                                               # взаимодействия с базой не будет, если все цены с "плохими" значениями

      filtered.each do |bp|                                                                                             # bp = {uom_id, season_id, currency_id, duration, value, discount}
        bp[:boat_id]    = boat.id
        bp[:is_orig]    = true
        bp[:value]      = bp[:value].is_a?(String) ? bp[:value].split(/[, ]/).join : bp[:value]                         # с формы может прийти строка, сгруппированная по разрадям, вида "100,100,222"
        bp[:duration]   = format('%.1f', bp[:duration])
        bp[:discount]   = bp[:discount].to_f                                                                            # с формы может прийти пустая строка ''
        bp[:created_at] = '\'%s\'' % Time.now.to_s(:db)
        _build_other bp
      end

      filtered2 = _del_duplicates(filtered + @built_prices)
      return false if filtered2.size.zero?

      saved = nil
      ActiveRecord::Base.transaction do
        _delete_existing_records boat.id
        saved = _save filtered2
      end

      @is_for_rent = _detect_for_rent saved
      true
    end

    private

    def _model
      BoatPrice
    end

    def _fields_for_create
      @fields_for_create ||= %w'boat_id uom_id duration season_id is_orig discount currency_id value created_at'
    end


    def _delete_bad_values(prices)
      prices.delete_if do |price|
        price[:value].nil? || price[:value].to_f.zero? || format('%.1f', price[:duration].to_f) == '0.0'
      end
    end

    def _del_duplicates(params)                                                                                         # подстраховываемся, вдруг форма прислала что-то не то
      grouped  = params.group_by { |row| [row[:duration].to_f, row[:uom_id], row[:season_id], row[:currency_id]] }      # убираем из params массива дубликаты согласно уникальному индексу в db
      grouped.map {|_, v| v.first }
    end

    def _reset_ivars
      @is_for_rent = nil
      super
    end

    def _detect_for_rent(saved_prices)
      return 0 if saved_prices.nil?
      # noinspection RubySimplifyBooleanInspection
      !!saved_prices.select { |bp| !bp[:_delete] }.size.nonzero? ? 1 : 0
    end

  end
end