module Boats
  #
  # [spec/services/boat/boat_sale_price_save_service_spec.rb]
  #
  class BoatSalePricesSaveService < ::AbstractPricesService

    attr_reader :is_for_sale

    def self.perform(boat_id, price)
      new.perform boat_id, price
    end

    def perform(boat_id, price = nil)
      _reset_ivars
      return false if price.nil?

      sp = price.dup
      sp[:boat_id]    = boat_id
      sp[:is_orig]    = true
      sp[:value]      = sp[:value].is_a?(String) ? sp[:value].split(/[, ]/).join : sp[:value].to_i                      # с формы может прийти строка, сгруппированная по разрадям, вида "100,100,222" + тут же превращаем nil в 0
      sp[:discount]   = sp[:discount].present? ? sp[:discount] : 0
      sp[:created_at] = '\'%s\'' % Time.now.to_s(:db)
      _build_other sp

      filtered2 = @built_prices + [sp]
      saved     = nil
      ActiveRecord::Base.transaction do
        _delete_existing_records boat_id
        saved = _save filtered2
      end

      @is_for_sale = _detect_for_sale saved
      true
    end

    private

    def _model
      BoatSalePrice
    end

    def _fields_for_create
      @fields_for_create ||= %w'boat_id currency_id value discount is_orig created_at'
    end

    def _reset_ivars
      @is_for_sale = nil
      super
    end

    def _detect_for_sale(saved_prices)
      return 0 if saved_prices.nil?
      # noinspection RubySimplifyBooleanInspection
      !!saved_prices.select { |bp| bp[:value].to_i != 0 }.size.nonzero? ? 1 : 0
    end

  end
end