module Lease
  #
  # Если заявка создаётся НЕ со страницы лодки - используется этот сервис
  #
  class CreateBroadcastInquiryService

    attr_reader :inquiry, :errors

    def initialize
      _reset_instance_variables
    end

    def perform(client_account, params)
      _reset_instance_variables

      @inquiry = ::Lease::Inquiry.new
      _assign_attributes(client_account, params)
      return false unless @inquiry.valid?

      inquiry_profile = ::Lease::InquiryProfile.new inquiry: @inquiry

      @inquiry.transaction do
        @inquiry.save!
        inquiry_profile.save!
      end

      if params[:uploaded_license_file].present?
        ::Lease::LicenseService.new.process_license_file(client_account, params[:uploaded_license_file])
      end

      true
    end

    protected

    def _assign_attributes(client_account, params)
      @inquiry.client_account      = client_account
      @inquiry.address             = params[:address]
      @inquiry.latitude            = params[:latitude]
      @inquiry.longitude           = params[:longitude]
      @inquiry.from                = params[:from]
      @inquiry.to                  = params[:to]
      @inquiry.name                = params[:name]
      @inquiry.phone_number        = params[:phone_number]
      @inquiry.is_skippered        = params[:is_skippered]
      @inquiry.comments            = params[:comments]
      @inquiry.need_transfer       = params[:need_transfer]
      @inquiry.watersports         = params[:watersports]
      @inquiry.guests              = params[:guests]
      @inquiry.kids                = params[:kids]
      @inquiry.boat_type_ids       = params[:boat_types] # fuckin' rent_form.rb
      @inquiry.staying_overnight   = params[:staying_overnight]
      @inquiry.hours_per_day       = [params[:hours_per_day]].join(',')

      if params[:my_price].present? && !params[:my_price].to_i.zero?
        @inquiry.my_price          = params[:my_price]
        @inquiry.my_price_currency = params[:my_price_currency]
        @inquiry.is_my_price       = true
      end
    end

    def _reset_instance_variables
      @errors  = { }
      @inquiry = nil
    end

  end
end
