module Lease

  class FindSuitsService

    attr_reader :errors,              # результат: ошибки
                :log,                 # результат: лог работы сервиса
                :emails,              # результат: почта владельцев лодок
                :aboats_ids           # результат: айдишники лодок с хозяевами. Эти айдишники вписаны в таблицу suits

    def initialize
      _reset_ivars
    end

    # вернёт false, если:
    #   * нет подходящих под заявку юзеров
    #   * нет подходящих под заявку лодок
    def perform(inquiry, personal_boat_id = nil)
      _reset_ivars
      @inquiry  = inquiry

      locs1      = BoatLocation.near([inquiry.latitude, inquiry.longitude], 300, units: :km, order: false)              # сначала ищем все лодки в Ш-радиусе (300км)
      locs2      = locs1.select { |loc| loc.radius >= loc.distance }                                                     # потом выбираем только те лодки, которые охватывают эту точку заявки

      boats_ids = locs2.map(&:boat_id).uniq
      boats     = Boat.where(id: boats_ids).where(boat_type_id: inquiry.boat_type_ids).where(for_rent: 1).where(state: ::Boat::STATE_APPROVED)
      boats_ids = boats.map(&:id)

      _log 'inquiry_id = ',          inquiry.id
      _log 'locs sql = ',            locs1.to_sql
      _log 'boats sql = ',           boats.to_sql
      _log 'result 1: boats_ids = ', boats_ids

      find_users boats_ids, personal_boat_id                                                                            # 3) фильтруем найденные лодки - выбираем только те, которые имеют хозяев
    end

    private

    def find_users(boat_ids, personal_boat_id)
      if boat_ids.count.zero?
        errors << 'BOATS NOT FOUND: inquiry = %s' % @inquiry.attributes
        _log 'BOATS NOT FOUND: inquiry =', @inquiry.attributes
        return false
      end

      # находим те лодки, у которых есть явные хозяева
      sql = '
      SELECT
                    users.id,
          boat_users.boat_id,
                 users.email,
             users.is_notify
      FROM users
      INNER JOIN boat_users ON boat_users.user_id = users.id
      WHERE boat_users.boat_id IN (%s)' % (boat_ids * ',')
      result = ActiveRecord::Base.connection.execute(sql)                                                               # for example: [[2977, 4675, "diego.s@gmx.com", 1], [2978, 4676, "dohomota@yandex.ru", 1]]

      # (22,55,509),(22,63,531),... - собираем значения для вставки в другой sql-запрос (создающий Suits)
      # values = result.map { |record| '(%s)' % ([@inquiry.id, record[0], record[1]] * ',') } # 0-user_id, 1-boat_id
      values = result.map do |record|
        @aboats_ids << record[1]
        '(%s)' % ([@inquiry.id, record[0], record[1]] * ',') # 0-user_id, 1-boat_id
      end

      _log 'boats with owners sql = ',    sql
      _log '(inq,user,boat) result = ', values

      unless values.present?
        errors << 'SUITS NOT FOUND: inquiry = %s, boat_ids = [%s]' % [@inquiry.attributes, boat_ids * ',']
        _log 'SUITS NOT FOUND: inquiry = ', @inquiry.attributes
        _log 'SUITS NOT FOUND: boat_ids = ', boat_ids * ','
        return false
      end

      # пишем в таблицу Suits подходящие под Заявку варианты
      sql2 = 'INSERT IGNORE INTO %s(inquiry_id, user_id, boat_id) VALUES%s' % [::Lease::Suit.table_name, values * ',']
      ActiveRecord::Base.connection.execute(sql2)

      @emails = if personal_boat_id.present?
                  # уведомляем только 1 хозяина лодки, на которую была оставлена персональная заявка
                  [result.select { |record| record[1] == personal_boat_id }.flatten[2]] # 2-users.email                 # for example: ['diego.s@gmx.com']
                else
                  # уведомляем всех агентов, подходящих под заявку
                  result.map do |record|
                    record[3].zero? ? nil : record[2] # 3-is_notify, 2-email
                  end.compact.uniq
                end

      _log 'insert ignore into suits sql =', sql2
      _log 'collect user emails = ',         @emails

      uc_values = if personal_boat_id.present?
                    # кружок рисуем только владельцу Целевой лодки, на которую была оставлена заявка
                    user_id = result.select { |record| record[1] == personal_boat_id }.flatten[0] # 0-user_id
                    ['(%s,%s)' % [user_id, @inquiry.id]]
                  else
                    # кружок рисуем всем подходящим под заявку агентам
                    result.map { |record| '(%s)' % ([record[0], @inquiry.id] * ',') } # 0-user_id
                  end

      # запишем счётчики (те, кому надо, увидят кружки в интерфейсе)
      sql3 = 'INSERT IGNORE INTO %s(user_id, inquiry_id) VALUES%s' % [::Lease::UsersInquiriesCounters.table_name, uc_values * ',']
      ActiveRecord::Base.connection.execute sql3

      _log 'insert counters = ', sql3

      puts @log * "\n"
      true
    end

    def _reset_ivars
      @errors     = []
      @emails     = []
      @log        = []
      @aboats_ids = []
    end

    def _log(key, value)
      @log << key
      @log << value
      @log << "\n"
    end
  end
end
