module Users
  class FindOrGenerateEmailTokenService

    attr_reader :token, :generated

    def perform(user_id, bid_id, token_subj_id)
      _reset_ivars

      new_token   = _find_token user_id, bid_id, token_subj_id
      new_token ||= _generate_token user_id, bid_id, token_subj_id
      @token      = new_token

      true
    end

    private

    # @return [Nil]
    # @return [String]
    #
    def _find_token(user_id, bid_id, token_subj_id)
      return unless user_id.present? && bid_id.present?

      sql = 'SELECT user_email_tokens.token FROM user_email_tokens
      INNER JOIN user_email_tokens_bids ON user_email_tokens.id = user_email_tokens_bids.token_id
      WHERE user_email_tokens_bids.bid_id = %s
      AND   user_email_tokens.user_id = %s
      AND   user_email_tokens.subj_id = %s
      LIMIT 1 ' % [bid_id, user_id, token_subj_id]

      res = ::ActiveRecord::Base.connection.exec_query(sql)
      res.as_json.dig(0, 'token')
    end

    # Генерация нужна только для тех случаев,
    # когда были отправлены письма без токенов ДО того момента,
    # когда токены были введены. Т.е. сейчас, по сути дела,
    # этот метод никогда не вызывается.
    #
    # @return [String]
    #
    def _generate_token(user_id, bid_id, token_subj_id)
      @generated = true
      user       = ::User.find user_id
      ::Users::GenerateEmailTokenService.call user, token_subj_id, bid_id
    end

    def _reset_ivars
      @token     = nil
      @generated = false
    end
  end
end
