# Когда база с боевого используется процессе разработки,
# вместо того, чтобы искать изображения локально,
# отправляем запрос на боевой сервер.
#
# ::usage::
#
# Rails.application.configure do
#   config.middleware.use ImagesProxy
# end
#
# ::dependency::
#
# gem 'rack-proxy'
#
#
if defined?(Rack::Proxy)
  class ImagesProxy < Rack::Proxy

    NEED_FAKE = /(system|ckeditor)/i      # в урле должно содержаться это, чтобы запрос ушёл на боевой
    HOST      = 'getboat.com:80'.freeze   # боевой адрес + порт

    def initialize(app)
      @app = app
    end

    def call(env)
      original_host = env['HTTP_HOST']
      rewrite_env(env)
      if env['HTTP_HOST'] != original_host
        perform_request(env)
      else
        # just regular
        @app.call(env)
      end
    end

    def rewrite_env(env)
      path = Rack::Request.new(env).path
      if path =~ NEED_FAKE && !File.exists?(path)
        env['HTTP_HOST']   = HOST
        env['SERVER_PORT'] = 80
      else
        # do nothing
      end
      env
    end
  end

end